@@ -32,10 +32,11 @@
 
 __copyright__ = 'Copyright (C) 2009, Purdue University'
 __license__ = 'BSD'
-__version__ = '0.14'
+__version__ = '0.15'
 
 
 from config import Config
+import constants
 import core
 from core import Core
 import core_helpers
@@ -44,8 +45,11 @@ from errors import CoreError
 import embedded_files
 
 
+SERVER_CONFIG_FILE_LOCATION = constants.SERVER_CONFIG_FILE_LOCATION
+
+
 __all__ = ['CoreError', 'core', 'Core', 'core_helpers', 'CoreHelpers',
-           'Config', 'embedded_files']
+           'Config', 'embedded_files', 'SERVER_CONFIG_FILE_LOCATION']
 
 
 # vi: set ai aw sw=2:
