@@ -32,7 +32,13 @@
 
 __copyright__ = 'Copyright (C) 2009, Purdue University'
 __license__ = 'BSD'
-__version__ = '0.14'
+__version__ = '0.15'
+
+
+# These are reserved ipv6 addresses.
+RESERVED_IPV6 = ('::', '::1/128', 'fe80::/10', 'ff01::1', 'ff02::1', 'ff01::2',
+                 'ff05::2', '3fff::/16', '2000::/16', '::/8')
+RESERVED_IPV4 = ('223/8', '224/3')
 
 
 # This is the default TTL for all records inserted into the database.
@@ -62,6 +68,12 @@ ACCESS_RIGHTS = ['rw', 'r']
 # it is important not to overwrite these tables when doing a partial replay
 TABLES_NOT_AUDIT_LOGGED = ['audit_log', 'locks']
 
+# This is a list of record types that can be indexed by IP address.
+RECORD_TYPES_INDEXED_BY_IP = ['ptr', 'a', 'aaaa']
+
+# This is the default config file location for roster server
+SERVER_CONFIG_FILE_LOCATION = '/etc/roster/roster_server.conf'
+
 # The SUPPORTED_METHODS hash contains a hash for every supported method.
 # 'check' indicates whether the target zone/IP range should be checked.
 # 'write' indicates whether the method requires write access.
@@ -512,6 +524,16 @@ TABLES = {
                           'success': 'IntBool',
                           'audit_log_timestamp': 'DateTime'},
     'locks':
-        {'lock_name': 'UnicodeString', 'locked': 'IntBool'}}
+        {'lock_name': 'UnicodeString', 'locked': 'IntBool'},
+
+    'ipv4_index':
+        {'ipv4_dec_address': 'UnsignedInt',
+         'ipv4_index_record_id': 'UnsignedInt'},
+
+    'ipv6_index':
+        {'ipv6_dec_upper': 'UnsignedInt',
+         'ipv6_dec_lower': 'UnsignedInt',
+         'ipv6_index_record_id': 'UnsignedInt'}}
+
 
 # vi: set ai aw sw=2:
