@@ -32,7 +32,7 @@
 
 __copyright__ = 'Copyright (C) 2009, Purdue University'
 __license__ = 'BSD'
-__version__ = '0.14'
+__version__ = '0.15'
 
 
 import constants
@@ -46,9 +46,6 @@ import math
 
 import IPy
 
-class RecordsBatchError(errors.CoreError):
-  pass
-
 class CoreHelpers(object):
   """Library of helper functions that extend the core functions."""
   def __init__(self, core_instance):
@@ -79,6 +76,150 @@ class CoreHelpers(object):
   def ExpandIPV6(self, ip_address):
     return helpers_lib.ExpandIPV6(ip_address)
 
+  def GetViewsByUser(self, user):
+    """Lists view names available to given user
+
+    Inputs:
+        user: string of user name
+
+    Outputs:
+        list: list of view name strings
+    """
+    views = set([])
+    users_dict = self.db_instance.GetEmptyRowDict('users')
+    users_dict['user_name'] = user
+    user_group_assignments_dict = self.db_instance.GetEmptyRowDict(
+        'user_group_assignments')
+    groups_dict = self.db_instance.GetEmptyRowDict('groups')
+    forward_zone_permissions_dict = self.db_instance.GetEmptyRowDict(
+        'forward_zone_permissions')
+    zones_dict = self.db_instance.GetEmptyRowDict('zones')
+    zone_view_assignments_dict = self.db_instance.GetEmptyRowDict(
+        'zone_view_assignments')
+    self.db_instance.StartTransaction()
+    try:
+      joined_list = self.db_instance.ListRow(
+          'users', users_dict, 'user_group_assignments',
+          user_group_assignments_dict,
+          'groups', groups_dict, 'forward_zone_permissions',
+          forward_zone_permissions_dict, 'zones', zones_dict,
+          'zone_view_assignments', zone_view_assignments_dict)
+    finally:
+      self.db_instance.EndTransaction()
+    for view_dict in joined_list:
+      views.add(view_dict['zone_view_assignments_view_dependency'].split(
+          '_dep')[0])
+    return views
+
+  def GetCIDRBlocksByView(self, view, user):
+    """Lists CIDR blocks available to given view
+
+    Inputs:
+        view: string of view name
+        user: string of user name
+
+    Outputs:
+        list: list of cidr block strings
+    """
+    cidrs = set([])
+    users_dict = self.db_instance.GetEmptyRowDict('users')
+    users_dict['user_name'] = user
+    views_dict = self.db_instance.GetEmptyRowDict('views')
+    views_dict['view_name'] = view
+
+    user_group_assignments_dict = self.db_instance.GetEmptyRowDict(
+        'user_group_assignments')
+    groups_dict = self.db_instance.GetEmptyRowDict('groups')
+    reverse_range_permissions_dict = self.db_instance.GetEmptyRowDict(
+        'reverse_range_permissions')
+    zones_dict = self.db_instance.GetEmptyRowDict('zones')
+    reverse_range_zone_assignments_dict = self.db_instance.GetEmptyRowDict(
+        'reverse_range_zone_assignments')
+    zone_view_assignments_dict = self.db_instance.GetEmptyRowDict(
+        'zone_view_assignments')
+    self.db_instance.StartTransaction()
+    try:
+      joined_list = self.db_instance.ListRow(
+          'views', views_dict,
+          'users', users_dict, 'user_group_assignments',
+          user_group_assignments_dict,
+          'groups', groups_dict, 'reverse_range_permissions',
+          reverse_range_permissions_dict, 'zones', zones_dict,
+          'zone_view_assignments', zone_view_assignments_dict,
+          'reverse_range_zone_assignments', reverse_range_zone_assignments_dict)
+    finally:
+      self.db_instance.EndTransaction()
+    for cidr_dict in joined_list:
+      cidrs.add(cidr_dict['reverse_range_zone_assignments_cidr_block'])
+    return cidrs
+
+  def GetAssociatedCNAMEs(self, hostname, view_name, zone_name,
+                          recursive=False):
+    """Lists cname's by assignment hostname.
+
+    Inputs:
+      hostname: string of hostname
+      view_name: string of view name
+      zone_name: string of zone name
+
+    Outputs:
+      list: list of found cname dictionaries
+    """
+    record_arguments_record_assignments_dict = (
+        self.db_instance.GetEmptyRowDict(
+            'record_arguments_records_assignments'))
+    zone_view_assignments_dict = self.db_instance.GetEmptyRowDict(
+        'zone_view_assignments')
+    zone_view_assignments_dict['zone_view_assignments_zone_name'] = unicode(
+         zone_name)
+    zone_view_assignments_dict['zone_view_assignments_view_dependency'] = (
+         unicode('%s_dep' % view_name))
+
+    record_arguments_record_assignments_dict[
+        'record_arguments_records_assignments_type'] = u'cname'
+    record_arguments_record_assignments_dict[
+        'argument_value'] = hostname
+    records_dict = self.db_instance.GetEmptyRowDict(
+        'records')
+    records_dict['record_type'] = u'cname'
+    records_dict['record_view_dependency'] = '%s_dep' % view_name
+    records_dict['record_zone_name'] = zone_name
+    self.db_instance.StartTransaction()
+    try:
+      found_records = self.db_instance.ListRow(
+          'records', records_dict,
+          'record_arguments_records_assignments',
+          record_arguments_record_assignments_dict,
+          'zone_view_assignments', zone_view_assignments_dict)
+    finally:
+      self.db_instance.EndTransaction()
+    cnames = []
+    for record in found_records:
+      new_record = {}
+      new_record['record_type'] = record[
+          'record_arguments_records_assignments_type']
+      new_record['zone_name'] = record['record_zone_name']
+      new_record['target'] = record['record_target']
+      new_record['ttl'] = record['record_ttl']
+      new_record['view_name'] = record[
+          'record_view_dependency'].rsplit('_dep')[0]
+      new_record['assignment_host'] = record['argument_value']
+      new_record['last_user'] = record['record_last_user']
+      new_record['zone_origin'] = record['zone_origin']
+      cnames.append(new_record)
+
+    if( not recursive ):
+      return cnames
+    new_cnames = []
+    for record in cnames:
+      new_cnames.extend(self.GetAssociatedCNAMEs(
+          '%s.%s' % (record['target'], record['zone_origin']),
+          record['view_name'], record['zone_name'], recursive=recursive))
+    cnames.extend(new_cnames)
+    del new_cnames
+    del found_records
+    return cnames
+
   def ListLatestNamedConfig(self, dns_server_set):
     """Lists the latest named config string given dns server set
 
@@ -168,7 +309,7 @@ class CoreHelpers(object):
     # Count number of characters in zone origin, add one to count the extra
     # period and remove that number of characters from the target.
     zone_origin_length = len(zone_origin) + 1
-    short_target = long_target[:-zone_origin_length:]
+    short_target = long_target[:-zone_origin_length]
 
     return (short_target, zone_assignment)
 
@@ -205,134 +346,230 @@ class CoreHelpers(object):
     self.core_instance.RemoveRecord(u'ptr', target, zone_name,
                                     record_args_dict, view_name, ttl)
 
-  def ListRecordsByCIDRBlock(self, cidr_block, view_name=None, zone_name=None):
-    """Lists records in user given cidr block.
+  def ListAvailableIpsInCIDR(self, cidr_block, num_ips=1, view_name=None,
+                           zone_name=None):
+    """Finds first available ips. Only lists as many IPs as are available.
+    Returns empty list if no IPs are available in given cidr block and a
+    truncated list if only a portion of IPs are available.
 
     Inputs:
       cidr_block: string of ipv4 or ipv6 cidr block
 
     Outputs:
-      dict: dictionary keyed by ip address example:
-            {u'192.168.1.7': {u'a': False, u'host': u'host5.',
-                              u'ptr': True, u'zone': u'test_zone',
-                              u'view': u'test_view2'},
-             u'192.168.1.5': {u'a': True, u'host': u'host3.',
-             u'ptr': True, u'zone': u'test_zone', u'view': u'test_view2'}}
+      list: list of strings of ip addresses
     """
-    user_cidr = IPy.IP(cidr_block)
-    record_type = u'a'
-    if( user_cidr.version() == 6 ):
-      record_type = u'aaaa'
-    zones = self.core_instance.ListZones()
-
-    ptr_record_list = []
-    fwd_record_list = []
-    zone_list = []
-
-    ptr_dict = self.db_instance.GetEmptyRowDict('records')
-    ptr_dict['record_type'] = u'ptr'
-    ptr_args_dict = self.db_instance.GetEmptyRowDict(
-        'record_arguments_records_assignments')
-
-    zone_dict = self.db_instance.GetEmptyRowDict(
-        'reverse_range_zone_assignments')
-
-    fwd_dict = self.db_instance.GetEmptyRowDict('records')
-    fwd_dict['record_type'] = record_type
-    if( view_name is not None and view_name.endswith('_dep')
-        or view_name == u'any' ):
-      fwd_dict['record_view_dependency'] = view_name
-      ptr_dict['record_view_dependency'] = view_name
-    elif( view_name is not None ):
-      fwd_dict['record_view_dependency'] = '%s_dep' % view_name
-      ptr_dict['record_view_dependency'] = '%s_dep' % view_name
-    fwd_args_dict = self.db_instance.GetEmptyRowDict(
-        'record_arguments_records_assignments')
-    fwd_args_dict['record_arguments_records_assignments_argument_name'] = (
-        u'assignment_ip')
-    self.db_instance.StartTransaction()
     try:
-      reverse_range_zone_assignments_db = (
-          self.db_instance.ListRow(
-              'reverse_range_zone_assignments', zone_dict))
-      for reverse_range_zone_assignment in reverse_range_zone_assignments_db:
-        db_zone = reverse_range_zone_assignment[
-            'reverse_range_zone_assignments_zone_name']
-        db_cidr = IPy.IP(reverse_range_zone_assignment[
-            'reverse_range_zone_assignments_cidr_block'])
-        if( user_cidr in db_cidr ):
-          zone_list = [db_zone]
-          break
-        if( db_cidr in user_cidr ):
-          zone_list.append(db_zone)
-      for zone in zone_list:
-        ptr_dict['record_zone_name'] = zone
-        ptr_record_list.extend(self.db_instance.ListRow(
-            'records', ptr_dict, 'record_arguments_records_assignments',
-            ptr_args_dict))
-      num_records = self.db_instance.TableRowCount('records')
-      ratio = num_records / float(user_cidr.len())
-      if( ratio > constants.RECORD_RATIO ):
-        for ip_address in user_cidr:
-          fwd_args_dict['argument_value'] = unicode(ip_address.strFullsize())
-          fwd_record_list.extend(self.db_instance.ListRow(
-              'records', fwd_dict, 'record_arguments_records_assignments',
-              fwd_args_dict))
+      cidr_block_ipy = IPy.IP(cidr_block)
+    except ValueError:
+      raise errors.CoreError('%s is not a valid cidr block' % cidr_block)
+    reserved_ips = []
+    if( cidr_block_ipy.version() == 6 ):
+      reserved = constants.RESERVED_IPV6
+    elif( cidr_block_ipy.version() == 4 ):
+      reserved = constants.RESERVED_IPV4
+    for cidr in reserved:
+      reserved_ips.append(IPy.IP(cidr))
+    for reserved_ip in reserved_ips:
+      if( IPy.IP(cidr_block) in reserved_ip ):
+        raise errors.CoreError('%s is in a reserved IP space' % cidr_block)
+    records = self.ListRecordsByCIDRBlock(cidr_block, view_name=view_name,
+                                          zone_name=zone_name)
+    taken_ips = []
+    avail_ips = []
+    for view in records:
+      for ip in records[view]:
+        taken_ips.append(ip)
+    count = 0L;
+    while( count < (cidr_block_ipy.len() - 1) ):
+      count += 1L
+      if( len(avail_ips) >= num_ips ):
+        break
+      if( cidr_block_ipy[count].strFullsize() not in taken_ips ):
+        avail_ips.append(cidr_block_ipy[count].strFullsize())
+    return avail_ips
+
+  def ListRecordsByCIDRBlock(self, cidr_block, view_name=None, zone_name=None):
+    """Lists records in a given cidr block.
+
+    Inputs:
+      cidr_block: string of ipv4 or ipv6 cidr block
+      view_name: string of the view
+      zone_name: string of the zone
+
+    Outputs:
+      dict: A dictionary Keyed by view, keyed by IP, listed by record.
+            example:
+                {u'test_view':
+                    {u'192.168.1.8':
+                        [{u'forward': True,
+                          u'host': u'host6.university.edu',
+                          u'zone': u'forward_zone',
+                          u'zone_origin': u'university.edu.'},
+                         {u'forward': False,
+                          u'host': u'host6.university.edu',
+                          u'zone': u'reverse_zone',
+                          u'zone_origin': u'1.168.192.in-addr.arpa.'}]}}
+    """
+    record_list = {}
+    try:
+      IPy.IP(cidr_block)
+    except ValueError:
+      raise errors.InvalidInputError(
+          'The CIDR block specified does not contain a valid IP: %s' % (
+          cidr_block))
+    cidr_block = IPy.IP(cidr_block).strFullsize(1)
+    if( cidr_block.find('/') != -1 ):
+      cidr_ip = IPy.IP(cidr_block.split('/')[0])
+      cidr_size = int(cidr_block.split('/')[1])
+    else:
+      cidr_ip = IPy.IP(cidr_block)
+      if( cidr_ip.version() == 4 ):
+        cidr_size = 32
+      elif( cidr_ip.version() == 6 ):
+        cidr_size = 128
       else:
-        fwd_record_list = self.db_instance.ListRow('records',
-            fwd_dict, 'record_arguments_records_assignments', fwd_args_dict)
-    finally:
-      self.db_instance.EndTransaction()
-    records_dict = {}
-    orig_view = view_name
-    for record in ptr_record_list:
-      view_name = orig_view
-      record_zone_name = record['record_zone_name']
-      db_view_name = record['record_view_dependency'].rsplit('_dep', 1)[0]
-      if( db_view_name != view_name and (db_view_name != u'any' and view_name is not None) ):
-        continue
-      if( zone_name and record_zone_name != zone_name ):
-        continue
-      if( view_name is None ):
-        view_name = db_view_name
-      if( db_view_name not in zones[record_zone_name]  ):
-        raise errors.CoreError('No zone view combination found for '
-                               '"%s" zone and "%s" view.' % (
-                                   record_zone_name, db_view_name))
-      zone_origin = zones[record_zone_name][db_view_name]['zone_origin']
-      reverse_ip_address = '%s.%s' % (record['record_target'], zone_origin)
-      ip_address = self.UnReverseIP(reverse_ip_address)
-      if( IPy.IP(ip_address) in user_cidr ):
-        if( not db_view_name in records_dict ):
-          records_dict[db_view_name] = {}
-        if( not ip_address in records_dict[db_view_name] ):
-          records_dict[db_view_name][ip_address] = []
-        records_dict[db_view_name][ip_address].append({
-            u'forward': False, u'host': record['argument_value'].rstrip('.'),
-            u'zone': record['record_zone_name'], 'zone_origin': zone_origin})
-    for record in fwd_record_list:
-      view_name = orig_view
-      ip_address = record['argument_value']
-      record_zone_name = record['record_zone_name']
-      db_view_name = record['record_view_dependency'].rsplit('_dep', 1)[0]
-      if( db_view_name != view_name and (db_view_name != u'any' and view_name is not None) ):
-        continue
-      if( zone_name and record_zone_name != zone_name ):
-        continue
-      if( view_name is None ):
-        view_name = db_view_name
-      zone_origin = zones[record_zone_name][db_view_name]['zone_origin']
-      if( IPy.IP(ip_address) in user_cidr ):
-        if( not db_view_name in records_dict ):
-          records_dict[db_view_name] = {}
-        if( not ip_address in records_dict[db_view_name] ):
-           records_dict[db_view_name][ip_address] = []
-        records_dict[db_view_name][ip_address].append({
-            u'forward': True, u'host': '%s.%s' % (
-                record['record_target'], zone_origin.rstrip('.')),
-            u'zone': record['record_zone_name'],
-            u'zone_origin': zone_origin})
-    return records_dict
+        raise errors.InvalidInputError(
+            'The CIDR block specified does not contain a valid IP: %s' % (
+            cidr_block))
+    records_dict = self.db_instance.GetEmptyRowDict('records')
+    zone_view_assignments_dict = self.db_instance.GetEmptyRowDict(
+        'zone_view_assignments')
+    zone_dict = self.db_instance.GetEmptyRowDict('zones')
+    record_arguments_records_assignments_dict = (
+        self.db_instance.GetEmptyRowDict(
+            'record_arguments_records_assignments'))
+
+    if( view_name is not None and 
+        view_name.endswith('_dep') or view_name == u'any' ):
+      records_dict['record_view_dependency'] = view_name
+    elif( view_name is not None ):
+      records_dict['record_view_dependency'] = '%s_dep' % view_name
+      
+    zone_dict['zone_name'] = zone_name
+    
+    if( cidr_ip.version() == 4 ):
+      decimal_ip = int( cidr_ip.strDec() )
+      decimal_ip_lower = (
+          (decimal_ip >> (32 - cidr_size) ) << (32 - cidr_size))
+      decimal_ip_upper = ( pow(2, 32 - cidr_size) - 1 ) | decimal_ip
+      self.db_instance.StartTransaction()
+      ip_index_dict = self.db_instance.GetEmptyRowDict('ipv4_index')
+      try:
+        record_list = self.db_instance.ListRow(
+            'ipv4_index', ip_index_dict,
+            'records', records_dict,
+            'zones', zone_dict,
+            'zone_view_assignments', zone_view_assignments_dict,
+            'record_arguments_records_assignments', 
+            record_arguments_records_assignments_dict,
+            column='ipv4_dec_address',
+            range_values=(decimal_ip_lower, decimal_ip_upper))
+      finally:
+        self.db_instance.EndTransaction()
+    elif( cidr_ip.version() == 6 ):
+      ip_index_dict = self.db_instance.GetEmptyRowDict('ipv6_index')
+      if( cidr_size >= 64 ):
+        try:
+          ip_index_dict[u'ipv6_dec_upper'] = int(cidr_ip.strHex(0)[:-16], 0)
+        except ValueError:
+          ip_index_dict[u'ipv6_dec_upper'] = 0
+        decimal_ip_lower = int('0x%s' % cidr_ip.strHex(0)[18:], 0)
+        decimal_ip_lower_lower = (
+            (decimal_ip_lower >> (128 - cidr_size)) <<
+            (128 - cidr_size))
+        decimal_ip_lower_upper = (
+            (pow(2,128 - cidr_size) - 1 ) | decimal_ip_lower)
+        column = 'ipv6_dec_lower'
+        range_values = (decimal_ip_lower_lower, decimal_ip_lower_upper)
+      elif( cidr_size < 64 ):
+        try:
+          decimal_ip_upper = int(cidr_ip.strHex()[:-16], 0)
+        except ValueError:
+          decimal_ip_upper = 0
+        decimal_ip_upper_lower = (
+            (decimal_ip_upper >> (64 - cidr_size)) << (64 - cidr_size))
+        decimal_ip_upper_upper = (
+            (pow(2,64 - cidr_size) - 1 ) | decimal_ip_upper)
+        column='ipv6_dec_upper'
+        range_values = (decimal_ip_upper_lower, decimal_ip_upper_upper)
+      self.db_instance.StartTransaction()
+      try:
+        record_list = self.db_instance.ListRow(
+            'ipv6_index', ip_index_dict,
+            'records', records_dict,
+            'zones', zone_dict,
+            'zone_view_assignments', zone_view_assignments_dict,
+            'record_arguments_records_assignments', 
+            record_arguments_records_assignments_dict,
+            column=column,
+            range_values=range_values)
+      finally:
+        self.db_instance.EndTransaction()
+     
+    ## Parse returned list
+    parsed_record_dict = {}
+    for index, record_entry in enumerate(record_list):
+      if( record_entry[u'record_type'] not in
+          constants.RECORD_TYPES_INDEXED_BY_IP ):
+        raise errors.IPIndexError('Record type not indexable by '
+                                  'IP: %s' % record_entry)
+      if( record_entry[u'record_view_dependency'].endswith('_dep') ):
+        record_view = record_entry[u'record_view_dependency'][:-4]
+      else:
+        record_view = record_entry[u'record_view_dependency']
+      if( record_view not in parsed_record_dict ):
+        parsed_record_dict[record_view] = {}
+      if( u'ipv4_dec_address' in record_entry ):
+        record_ip = u'%s' % (
+            IPy.IP(record_entry[u'ipv4_dec_address']).strNormal(1))
+        if( record_ip not in parsed_record_dict[record_view] ):
+          parsed_record_dict[record_view][record_ip] = []
+      elif( u'ipv6_dec_upper' in record_entry ):
+        decimal_ip = (
+            (record_entry[u'ipv6_dec_upper'] << 64) +
+            (record_entry[u'ipv6_dec_lower']) )
+        record_ip = u'%s' % IPy.IP(decimal_ip).strFullsize(0)
+        if( record_ip not in parsed_record_dict[record_view] ):
+          parsed_record_dict[record_view][record_ip] = []
+      else:
+        raise errors.IPIndexError(
+            'Record type unknown. Missing ipv4 or ipv6 dec index: %s' % (
+            record_entry))
+      record_item = {}
+      record_item['records_id'] = record_entry['records_id']
+      record_item['record_type'] = record_entry['record_type']
+      record_item['record_target'] = record_entry['record_target']
+      record_item['record_ttl'] = record_entry['record_ttl']
+      record_item['record_zone_name'] = record_entry['record_zone_name']
+      record_item[u'zone_origin'] = record_entry[u'zone_origin']
+      record_item['record_view_dependency'] = record_entry['record_view_dependency']
+      #record_item['record_last_updated'] = record_entry['record_last_updated']
+      record_item['record_last_user'] = record_entry['record_last_user']
+      if record_entry[u'record_view_dependency'].endswith('_dep'):
+        record_item[u'view_name'] = record_entry[u'record_view_dependency'][:-4]
+      else:
+        record_item[u'view_name'] = record_entry[u'record_view_dependency']
+      if( record_entry[u'record_type'] == u'a' or 
+          record_entry[u'record_type'] == u'aaaa' ):
+        record_item[u'forward'] = True
+        record_item[u'host'] = '%s.%s' % (
+            record_entry[u'record_target'],
+            record_entry[u'zone_origin'][:-1])
+        record_item[u'zone_origin'] = record_entry['zone_origin']
+        record_item[u'record_target'] = record_entry['record_target']
+        record_item[u'record_args_dict'] = {
+            'assignment_ip': record_entry['argument_value']}
+        parsed_record_dict[record_view][record_ip].append( record_item )
+      elif( record_entry[u'record_type'] == u'ptr' ):
+        record_item[u'zone_origin'] = record_entry['zone_origin']
+        record_item[u'record_target'] = record_entry['record_target']
+        record_item[u'forward'] = False
+        record_item[u'host'] = record_entry[u'argument_value'][:-1]
+        assignment_ip = helpers_lib.UnReverseIP(
+            '%s.%s' % (record_entry['record_target'],record_entry['zone_origin']))
+        record_item[u'record_args_dict'] = {'assignment_ip': assignment_ip}
+        parsed_record_dict[record_view][record_ip].insert(0, record_item )
+    return parsed_record_dict
 
   def ListNamedConfGlobalOptionsClient(self, option_id=None,
                                        dns_server_set=None, timestamp=None):
@@ -453,9 +690,13 @@ class CoreHelpers(object):
 
     Inputs:
       delete_records: list of dictionaries of records
-                      ex: {'record_type': 'a', 'record_target': 'target',
-                           'view_name': 'view', 'zone_name': 'zone',
-                           'record_arguments': {'assignment_ip': '1.2.3.4'}}
+                      ex: {'record_ttl': 3600, 'record_type': u'a',
+                          'records_id': 10, 'record_target': u'host1',
+                          'record_zone_name': u'forward_zone',
+                          'record_last_user': u'sharrell',
+                          'record_view_dependency': u'test_view_dep'}
+                          {'record_type': 'ptr', 'record_target': 'target',
+                          'view_name': 'view', 'zone_name': 'zone'}
       add_records: list of dictionaries of records
 
     Outputs:
@@ -472,74 +713,43 @@ class CoreHelpers(object):
       try:
         # REMOVE RECORDS
         for record in delete_records:
-          records_dict = self.db_instance.GetEmptyRowDict('records')
-          records_dict['record_type'] = record['record_type']
-          records_dict['record_target'] = record['record_target']
-          records_dict['record_zone_name'] = record['record_zone_name']
-          view_name = record['view_name']
-          if( not record['view_name'].endswith('_dep') and record[
-                'view_name'] != u'any'):
-            view_name = '%s_dep' % record['view_name']
-          changed_view_dep.append((view_name, record['record_zone_name']))
-          records_dict['record_view_dependency'] = view_name
-          self.user_instance.Authorize('RemoveRecord',
-              record_data =
-                  {'target': record['record_target'],
-                   'zone_name': record['record_zone_name'],
-                   'view_name': records_dict['record_view_dependency']},
+          record_dict = self.db_instance.GetEmptyRowDict('records')
+          self.user_instance.Authorize('RemoveRecord', 
+              record_data = {
+                  'target': record['record_target'],
+                  'zone_name': record['record_zone_name'],
+                  'view_name': record_dict['record_view_dependency']},
               current_transaction=True)
-
-          if( 'record_ttl' in record ):
-            records_dict['record_ttl'] = record['record_ttl']
-          args_list = []
-          for argument in record['record_arguments']:
-            if( record['record_arguments'][argument] is None ):
-              raise errors.CoreErrore('%s: "%s" cannot be None' % (
-                  record['record_target'], argument))
-            args_list.append(
-                {u'record_arguments_records_assignments_argument_name':
-                     argument,
-                 u'record_arguments_records_assignments_type':
-                     record['record_type'],
-                 u'argument_value': record['record_arguments'][argument],
-                 u'record_arguments_records_assignments_record_id': None})
-          args_search_list = []
-          record_ids = []
-          final_id = []
-          record_id_dict = {}
-          for arg in args_list:
-            args_search_list.append(self.db_instance.ListRow(
-                'record_arguments_records_assignments', arg))
-          for index, record_args in enumerate(args_search_list):
-            record_ids.append([])
-            for args_dict in record_args:
-              record_ids[index].append(args_dict[
-                  u'record_arguments_records_assignments_record_id'])
-          for id_list in record_ids:
-            for search_id in id_list:
-              if( search_id in record_id_dict ):
-                record_id_dict[search_id] += 1
-              else:
-                record_id_dict[search_id] = 1
-          for record_id in record_id_dict:
-            if( record_id_dict[record_id] == len(args_list) ):
-              final_id.append(record_id)
-          if( len(final_id) == 1 ):
-            records_dict['records_id'] = final_id[0]
-            new_records = self.db_instance.ListRow('records', records_dict)
-            rows_deleted = self.db_instance.RemoveRow('records', new_records[0])
-            if( not rows_deleted ):
-              raise RecordsBatchError(
-                  '%s: Record not found' % record['record_target'])
-            log_dict['delete'].append(record)
-            row_count += 1
+          record_dict['records_id'] = record['records_id']
+          record_dict['record_type'] = record['record_type']
+          record_dict['record_target'] = record['record_target']
+          record_dict['record_ttl'] = record['record_ttl']
+          if( record['record_view_dependency'].endswith('_dep') or 
+              record['record_view_dependency'] == u'any' ):
+            record_dict['record_view_dependency'] = record['record_view_dependency']
+          else:
+            record_dict['record_view_dependency'] = (
+                '%s_dep' % record['record_view_dependency'])
+          record_dict['record_zone_name'] = record['record_zone_name']
+          record_dict['record_last_user'] = record['record_last_user']
+          rows_deleted = self.db_instance.RemoveRow('records', record_dict)
+          log_dict['delete'].append(record)
+          row_count += 1
+          if( rows_deleted > 1 ):
+            raise errors.RecordsBatchError(
+                  'Record specification too broad, '
+                  'found %d matching records for %s.' % (
+                      rows_deleted, record_dict))
+          elif( rows_deleted == 0 ):
+            raise errors.RecordsBatchError(
+                  'No record found for :%s' % record_dict)
 
         # ADD RECORDS
         for record in add_records:
-          view_name = record['view_name']
-          if( not record['view_name'].endswith('_dep') and record[
-                'view_name'] != u'any'):
-            view_name = '%s_dep' % record['view_name']
+          view_name = record['record_view_dependency']
+          if( not record['record_view_dependency'].endswith('_dep') and record[
+                'record_view_dependency'] != u'any'):
+            view_name = '%s_dep' % record['record_view_dependency']
           self.user_instance.Authorize('MakeRecord', 
               record_data = {
                   'target': record['record_target'],
@@ -569,14 +779,14 @@ class CoreHelpers(object):
           if( record['record_type'] == 'cname' ):
             all_records = self.db_instance.ListRow('records', records_dict)
             if( len(all_records) > 0 ):
-              raise RecordsBatchError(
+              raise errors.RecordsBatchError(
                   'Record already exists with target %s.' % (
-                      record['record_target']))
+                  record['record_target']))
           records_dict['record_type'] = u'cname'
           cname_records = self.db_instance.ListRow('records', records_dict)
           if( len(cname_records) > 0 ):
-            raise RecordsBatchError('CNAME already exists with target %s.' % (
-                record['record_target']))
+            raise errors.RecordsBatchError('CNAME already exists with target '
+                                           '%s.' % (record['record_target']))
 
           record_args_assignment_dict = self.db_instance.GetEmptyRowDict(
               'record_arguments_records_assignments')
@@ -598,7 +808,7 @@ class CoreHelpers(object):
               if( record['record_arguments'][arg] != current_record[arg] ):
                 break
             else:
-              raise RecordsBatchError('Duplicate record found')
+              raise errors.RecordsBatchError('Duplicate record found')
 
 
           records_dict['record_type'] = record['record_type']
@@ -614,7 +824,13 @@ class CoreHelpers(object):
                                      record_argument_assignments_dict)
             log_dict['add'].append(record)
             row_count += 1
-
+          if( records_dict['record_type'] in
+              constants.RECORD_TYPES_INDEXED_BY_IP ):
+            self.core_instance._AddRecordToIpIndex(
+                records_dict['record_type'], records_dict['record_zone_name'],
+                records_dict['record_view_dependency'],
+                record_id, records_dict['record_target'],
+                record['record_arguments'])
         changed_view_dep = set(changed_view_dep)
         for view_dep_pair in changed_view_dep:
           self.core_instance._IncrementSoa(*view_dep_pair)
