@@ -66,7 +66,7 @@ Note: MySQLdb.Error can be raised in almost any function in this module. Please
 
 __copyright__ = 'Copyright (C) 2009, Purdue University'
 __license__ = 'BSD'
-__version__ = '0.14'
+__version__ = '0.15'
 
 
 import datetime
@@ -86,20 +86,7 @@ import errors
 import helpers_lib
 
 
-class TransactionError(errors.DbAccessError):
-  pass
-
-
-class UnexpectedDataError(data_validation.UnexpectedDataError):
-  pass
-
-
-class InvalidInputError(data_validation.InvalidInputError):
-  pass
-
-
-class MissingDataTypeError(data_validation.MissingDataTypeError):
-  pass
+DEBUG = False
 
 
 class dbAccess(object):
@@ -108,7 +95,8 @@ class dbAccess(object):
    """
 
   def __init__(self, db_host, db_user, db_passwd, db_name, big_lock_timeout,
-               big_lock_wait, thread_safe=True):
+               big_lock_wait, thread_safe=True, ssl=False, ssl_ca=None,
+               ssl_cert=None, ssl_key=None, ssl_capath=None, ssl_cipher=None):
     """Instantiates the db_access class.
 
     Inputs:
@@ -128,6 +116,12 @@ class dbAccess(object):
     self.db_name = db_name
     self.big_lock_timeout = big_lock_timeout
     self.big_lock_wait = big_lock_wait
+    self.ssl = ssl
+    self.ssl_settings = {'ca': ssl_ca, 'cert': ssl_cert, 'key': ssl_key}
+    if( ssl_capath ):
+      self.ssl_settings['capath'] = ssl_capath
+    if( ssl_cipher ):
+      self.ssl_settings['cipher'] = ssl_cipher
     self.transaction_init = False
     self.connection = None
     self.cursor = None
@@ -149,6 +143,20 @@ class dbAccess(object):
     if( self.connection is not None ):
       self.connection.close()
     self.connection = None
+
+
+  def cursor_execute(self, execution_string, values={}):
+    """This function allows for the capture of every mysql command that
+       is run in this class. 
+
+    Inputs:
+      execution_string: mysql command string
+      values: dictionary of values for mysql command
+    """
+    if( DEBUG == True ):
+      print execution_string % values
+    self.cursor.execute(execution_string, values)
+    
     
   def StartTransaction(self):
     """Starts a transaction.
@@ -178,19 +186,27 @@ class dbAccess(object):
 
     else:
       if( self.transaction_init ):
-        raise TransactionError('Cannot start new transaction last transaction '
-                               'not committed or rolled-back.')
+        raise errors.TransactionError('Cannot start new transaction last '
+                                      'transaction not committed or '
+                                      'rolled-back.')
 
     if( self.connection is not None ):
       try:
         self.cursor = self.connection.cursor(MySQLdb.cursors.DictCursor)
+        self.cursor_execute('DO 0') # NOOP to test connection
       except MySQLdb.OperationalError:
         self.connection = None
 
     if( self.connection is None ):
-      self.connection = MySQLdb.connect(host=self.db_host, user=self.db_user,
-                                        passwd=self.db_passwd, db=self.db_name,
-                                        use_unicode=True)
+      if( self.ssl ):
+          self.connection = MySQLdb.connect(
+              host=self.db_host, user=self.db_user, passwd=self.db_passwd,
+              db=self.db_name, use_unicode=True, charset='utf8',
+              ssl=self.ssl_settings)
+      else:
+          self.connection = MySQLdb.connect(
+              host=self.db_host, user=self.db_user, passwd=self.db_passwd,
+              db=self.db_name, use_unicode=True, charset='utf8')
       self.cursor = self.connection.cursor(MySQLdb.cursors.DictCursor)
 
     while_sleep = 0
@@ -198,7 +214,7 @@ class dbAccess(object):
     while( db_lock_locked ):
       time.sleep(while_sleep)
       try:
-        self.cursor.execute('SELECT `locked`, `lock_last_updated`, '
+        self.cursor_execute('SELECT `locked`, `lock_last_updated`, '
                             'NOW() as `now` from `locks` WHERE '
                             '`lock_name`="db_lock_lock"')
         rows = self.cursor.fetchall()
@@ -229,8 +245,8 @@ class dbAccess(object):
     """
     if( not self.thread_safe ):
       if( not self.transaction_init ):
-        raise TransactionError('Must run StartTansaction before '
-                               'EndTransaction.')
+        raise errors.TransactionError('Must run StartTansaction before '
+                                      'EndTransaction.')
 
     try:
       self.cursor.close()
@@ -264,11 +280,11 @@ class dbAccess(object):
     valid.
     """
     if( self.locked_db is True ):
-      raise TransactionError('Must unlock tables before re-locking them')
-    self.cursor.execute('UPDATE `locks` SET `locked`=1 WHERE '
+      raise errors.TransactionError('Must unlock tables before re-locking them')
+    self.cursor_execute('UPDATE `locks` SET `locked`=1 WHERE '
                         '`lock_name`="db_lock_lock"')
     time.sleep(self.big_lock_wait)
-    self.cursor.execute(
+    self.cursor_execute(
         'LOCK TABLES %s READ' % ' READ, '.join(self.ListTableNames()))
     self.locked_db = True
 
@@ -279,9 +295,9 @@ class dbAccess(object):
     valid. It also expects all tables to be locked.
     """
     if( self.locked_db is False ):
-      raise TransactionError('Must lock tables before unlocking them')
-    self.cursor.execute('UNLOCK TABLES')
-    self.cursor.execute('UPDATE `locks` SET `locked`=0 WHERE '
+      raise errors.TransactionError('Must lock tables before unlocking them')
+    self.cursor_execute('UNLOCK TABLES')
+    self.cursor_execute('UPDATE `locks` SET `locked`=0 WHERE '
                         '`lock_name`="db_lock_lock"')
     self.locked_db = False
 
@@ -294,6 +310,8 @@ class dbAccess(object):
     """
     cursor = self.connection.cursor()
     try:
+      if( DEBUG == True ):
+        print 'SELECT reserved_word FROM reserved_words'
       cursor.execute('SELECT reserved_word FROM reserved_words')
       rows = cursor.fetchall()
     finally:
@@ -318,9 +336,10 @@ class dbAccess(object):
       int: last insert id
     """
     if( not table_name in helpers_lib.GetValidTables() ):
-      raise InvalidInputError('Table name not valid: %s' % table_name)
+      raise errors.InvalidInputError('Table name not valid: %s' % table_name)
     if( not self.transaction_init ):
-      raise TransactionError('Must run StartTansaction before inserting.')
+      raise errors.TransactionError('Must run StartTansaction before '
+                                    'inserting.')
     if( self.data_validation_instance is None ):
       self.InitDataValidation()
     self.data_validation_instance.ValidateRowDict(table_name, row_dict) 
@@ -334,7 +353,7 @@ class dbAccess(object):
     query = 'INSERT INTO %s (%s) VALUES (%s)' % (table_name, 
                                                  ','.join(column_names),
                                                  ','.join(column_assignments))
-    self.cursor.execute(query, row_dict)
+    self.cursor_execute(query, row_dict)
     return self.cursor.lastrowid
 
   def TableRowCount(self, table_name):
@@ -352,11 +371,11 @@ class dbAccess(object):
     """
 
     if( not table_name in helpers_lib.GetValidTables() ):
-      raise InvalidInputError('Table name not valid: %s' % table_name)
+      raise errors.InvalidInputError('Table name not valid: %s' % table_name)
     if( not self.transaction_init ):
-      raise TransactionError('Must run StartTansaction before getting row '
-                             'count.')
-    self.cursor.execute('SELECT COUNT(*) FROM %s' % table_name)
+      raise errors.TransactionError('Must run StartTansaction before getting '
+                                    'row count.')
+    self.cursor_execute('SELECT COUNT(*) FROM %s' % table_name)
     row_count = self.cursor.fetchone()
     return row_count['COUNT(*)']
 
@@ -375,9 +394,9 @@ class dbAccess(object):
       int: number of rows affected
     """
     if( not table_name in helpers_lib.GetValidTables() ):
-      raise InvalidInputError('Table name not valid: %s' % table_name)
+      raise errors.InvalidInputError('Table name not valid: %s' % table_name)
     if( not self.transaction_init ):
-      raise TransactionError('Must run StartTansaction before deleting.')
+      raise errors.TransactionError('Must run StartTansaction before deleting.')
     if( self.data_validation_instance is None ):
       self.InitDataValidation()
     self.data_validation_instance.ValidateRowDict(table_name, row_dict) 
@@ -387,7 +406,7 @@ class dbAccess(object):
       where_list.append('%s=%s%s%s' % (k, '%(', k, ')s'))
 
     query = 'DELETE FROM %s WHERE %s' % (table_name, ' AND '.join(where_list))
-    self.cursor.execute(query, row_dict)
+    self.cursor_execute(query, row_dict)
     return self.cursor.rowcount
 
   def UpdateRow(self, table_name, search_row_dict, update_row_dict):
@@ -408,9 +427,9 @@ class dbAccess(object):
       int: number of rows affected
     """
     if( not table_name in helpers_lib.GetValidTables() ):
-      raise InvalidInputError('Table name not valid: %s' % table_name)
+      raise errors.InvalidInputError('Table name not valid: %s' % table_name)
     if( not self.transaction_init ):
-      raise TransactionError('Must run StartTansaction before deleting.')
+      raise errors.TransactionError('Must run StartTansaction before deleting.')
     if( self.data_validation_instance is None ):
       self.InitDataValidation()
     self.data_validation_instance.ValidateRowDict(table_name, search_row_dict,
@@ -433,7 +452,7 @@ class dbAccess(object):
 
     query = 'UPDATE %s SET %s WHERE %s' % (table_name, ','.join(query_updates),
                                            ' AND '.join(query_searches))
-    self.cursor.execute(query, combined_dict)
+    self.cursor_execute(query, combined_dict)
     return self.cursor.rowcount
 
   def ListRow(self, *args, **kwargs):
@@ -444,10 +463,11 @@ class dbAccess(object):
     Inputs:
       args: pairs of string of table name and dict of rows
       kwargs: lock_rows: default False
-              date_column: column to search date range on, if using multiple
-                           tables, date column must be in the first table
+              column: column to search range on, if using multiple
+                           tables, the column must be in the first table
                            in args.
-              date_range: range of dates to search for on date_column
+              range_values: range tuple of values to search within for on column
+              is_date: boolean of if range is of dates
 
       example usage: ListRow('users', user_row_dict,
                              'user_group_assignments', user_assign_row_dict,
@@ -468,7 +488,8 @@ class dbAccess(object):
                    'user_group_assignments_user_name: 'sharrell'})
     """
     if( not self.transaction_init ):
-      raise TransactionError('Must run StartTansaction before getting data.')
+      raise errors.TransactionError('Must run StartTansaction before getting '
+                                    'data.')
     if( self.data_validation_instance is None ):
       self.InitDataValidation()
 
@@ -476,37 +497,45 @@ class dbAccess(object):
     tables = {}
     table_names = []
     lock_rows = False
-    date_column = None
-    date_range = ()
+    column = None
+    range_values = ()
+    is_date = None
     if( kwargs ):
       if( 'lock_rows' in kwargs ):
         lock_rows = kwargs['lock_rows']
         del kwargs['lock_rows']
-      if( 'date_column' in kwargs ):
-        date_column = kwargs['date_column']
-        del kwargs['date_column']
-      if( 'date_range' in kwargs ):
-        date_range = kwargs['date_range']
-        del kwargs['date_range']
-      if( bool(date_column) ^ bool(date_range) ):
-        raise InvalidInputError('If date_column or date_range is specified '
-                                'both are needed')
+      if( 'column' in kwargs ):
+        column = kwargs['column']
+        del kwargs['column']
+      if( 'range_values' in kwargs ):
+        range_values = kwargs['range_values']
+        del kwargs['range_values']
+      if( 'is_date' in kwargs ):
+        is_date = kwargs['is_date']
+        del kwargs['is_date']
+      if( column is None and is_date is not None ):
+        raise errors.InvalidInputError('If is_date is specified you must '
+                                       'specify column and range')
+      if( bool(column) ^ bool(range_values) ):
+        raise errors.InvalidInputError('If column or range is specified '
+                                       'both are needed')
       if( kwargs ):
-        raise InvalidInputError('Found unknown option(s): %s' % kwargs.keys())
+        raise errors.InvalidInputError('Found unknown option(s): '
+                                       '%s' % kwargs.keys())
 
     if( not args ):
-      raise InvalidInputError('No args given, must at least have on pair of '
-                              'table name and row dict')
+      raise errors.InvalidInputError('No args given, must at least have on '
+                                     'pair of table name and row dict')
     if( len(args) % 2 ):
-      raise InvalidInputError('Number of unnamed args is not even. Args '
-                              'should be entered in pairs of table name and '
-                              'row dict.')
+      raise errors.InvalidInputError('Number of unnamed args is not even. Args '
+                                     'should be entered in pairs of table name '
+                                     'and row dict.')
     count = 0
     for arg in args:
       count += 1
       if( count % 2 ):
         if( not arg in valid_tables ):
-          raise InvalidInputError('Table name not valid: %s' % arg)
+          raise errors.InvalidInputError('Table name not valid: %s' % arg)
         current_table_name = arg
       else:
         # do checking in validate row dict to check if it is a dict
@@ -516,22 +545,28 @@ class dbAccess(object):
         tables[current_table_name] = arg
         table_names.append(current_table_name)
 
-    if( date_range ):
-      if( date_column not in args[1] ):
-        raise InvalidInputError('Column %s from date_column not found in row'
-                                'dictionary: %s' % (date_column, args[1]))
+    if( range_values ):
+      if( column not in args[1] ):
+        raise errors.InvalidInputError('Column %s not found in row'
+                                       'dictionary: %s' % (column, args[1]))
 
-      if( constants.TABLES[args[0]][date_column] != 'DateTime' ):
-        raise InvalidInputError('date_column: %s in table %s is not a'
-                                'DateTime type' % (date_column, args[0]))
-      for date in date_range:
-        if( not self.data_validation_instance.isDateTime(date) ):
-          raise InvalidInputError('Date: %s from date_range is not a valid '
-                                  'datetime object' % date)
+      if( is_date ):
+        if( constants.TABLES[args[0]][column] != 'DateTime' ):
+          raise errors.InvalidInputError('column: %s in table %s is not a'
+                                         'DateTime type' % (column, args[0]))
+        for date in range_values:
+          if( not self.data_validation_instance.isDateTime(date) ):
+            raise errors.InvalidInputError('Date: %s from range is not a valid '
+                                           'datetime object' % date)
+      else:
+        for value in range_values:
+          if( not self.data_validation_instance.isUnsignedInt(value) ):
+            raise errors.InvalidInputError('Range must be int if is_date '
+                                           'is not set')
     query_where = []
     if( len(tables) > 1 ):
       if( not self.foreign_keys ):
-        self.cursor.execute('SELECT table_name, column_name, '
+        self.cursor_execute('SELECT table_name, column_name, '
                             'referenced_table_name, referenced_column_name '
                             'FROM information_schema.key_column_usage WHERE '
                             'referenced_table_name IS NOT NULL AND '
@@ -546,8 +581,8 @@ class dbAccess(object):
                              '%(referenced_table_name)s.'
                              '%(referenced_column_name)s)' % key)
       if( not query_where ):
-        raise InvalidInputError('Multiple tables were passed in but no joins '
-                                'were found')
+        raise errors.InvalidInputError('Multiple tables were passed in but no '
+                                       'joins were found')
     column_names = []
     search_dict = {}
     for table_name, row_dict in tables.iteritems():
@@ -557,11 +592,11 @@ class dbAccess(object):
           search_dict[k] = v
           query_where.append('%s%s%s%s' % (k, '=%(', k, ')s'))
 
-    if( date_range ):
-      search_dict['begin_date'] = date_range[0]
-      search_dict['end_date'] = date_range[1]
-      query_where.append('%s%s%s%s' % (date_column, '>=%(begin_date)s AND ',
-                                       date_column, '<=%(end_date)s'))
+    if( range_values ):
+      search_dict['start'] = range_values[0]
+      search_dict['end'] = range_values[1]
+      query_where.append('%s%s%s%s' % (column, '>=%(start)s AND ',
+                                       column, '<=%(end)s'))
 
     query_end = ''
     if( query_where ):
@@ -573,7 +608,7 @@ class dbAccess(object):
                                       ','.join(table_names),
                                       query_end)
 
-    self.cursor.execute(query, search_dict)
+    self.cursor_execute(query, search_dict)
     return self.cursor.fetchall()
 
   def GetEmptyRowDict(self, table_name):
@@ -595,7 +630,7 @@ class dbAccess(object):
     """
     row_dict = helpers_lib.GetRowDict(table_name)
     if( not row_dict ):
-      raise InvalidInputError('Table name not valid: %s' % table_name)
+      raise errors.InvalidInputError('Table name not valid: %s' % table_name)
     for k in row_dict.iterkeys():
       row_dict[k] = None
     return row_dict
@@ -634,7 +669,7 @@ class dbAccess(object):
 
     record_arguments_dict = {}
     if( not record_arguments ):
-      raise InvalidInputError('Unknown record type: %s' % record_type)
+      raise errors.InvalidInputError('Unknown record type: %s' % record_type)
     for record_argument in record_arguments:
       record_arguments_dict[record_argument['argument_name']] = (
           record_argument['argument_data_type'])
@@ -675,8 +710,9 @@ class dbAccess(object):
     """
     record_type_dict = self.GetRecordArgsDict(record_type)
     if( not set(record_type_dict.keys()) == set(record_args_dict.keys()) ):
-      raise InvalidInputError('dict for record type %s should have these '
-                              'keys: %s' % (record_type, record_type_dict))
+      raise errors.InvalidInputError('dict for record type %s should have '
+                                     'these keys: %s' % (record_type,
+                                     record_type_dict))
     if( self.data_validation_instance is None ):
       self.InitDataValidation()
     
@@ -684,17 +720,17 @@ class dbAccess(object):
     for record_arg_name in record_args_dict.keys():
       if( not 'is%s' % record_type_dict[record_arg_name] in
           data_validation_methods ):
-        raise MissingDataTypeError('No function to check data type %s' %
-                                   record_type_dict[record_arg_name])
+        raise errors.MissingDataTypeError('No function to check data type %s' %
+                                          record_type_dict[record_arg_name])
       if( none_ok and record_args_dict[record_arg_name] is None ):
         continue
 
       if( not getattr(self.data_validation_instance, 'is%s' %
           record_type_dict[record_arg_name])(
               record_args_dict[record_arg_name]) ):
-        raise UnexpectedDataError('Invalid data type %s: %s' %
-                                  (record_type_dict[record_arg_name],
-                                   record_args_dict[record_arg_name]))
+        raise errors.UnexpectedDataError('Invalid data type %s: %s' % (
+                                         record_type_dict[record_arg_name],
+                                         record_args_dict[record_arg_name]))
   def ListTableNames(self):
     """Lists all tables in the database.
 
@@ -702,7 +738,7 @@ class dbAccess(object):
       List: List of tables
     """
     query = 'SHOW TABLES'
-    self.cursor.execute(query)
+    self.cursor_execute(query)
     tables = self.cursor.fetchall()
     table_list = []
     for table_dict in tables:
@@ -716,7 +752,7 @@ class dbAccess(object):
     Outputs:
       datetime: current time in the database
     """
-    self.cursor.execute('SELECT NOW()')
+    self.cursor_execute('SELECT NOW()')
     return self.cursor.fetchone()['NOW()']
 
   def CreateRosterDatabase(self, schema=None):
@@ -752,7 +788,7 @@ class dbAccess(object):
     for line in execute_lines:
       self.StartTransaction()
       try:
-        self.cursor.execute(line)
+        self.cursor_execute(line)
       finally:
         self.EndTransaction()
 
@@ -767,20 +803,20 @@ class dbAccess(object):
                   table as values.
     """
     table_data = {}
-    self.cursor.execute('SHOW TABLES')
+    self.cursor_execute('SHOW TABLES')
     table_names = self.cursor.fetchall()
-    self.cursor.execute('SET OPTION SQL_QUOTE_SHOW_CREATE=1')
+    self.cursor_execute('SET OPTION SQL_QUOTE_SHOW_CREATE=1')
     for table_name in table_names:
       table_name = table_name.values()[0]
       table_data[table_name] = {}
-      self.cursor.execute('SHOW CREATE TABLE %s' % table_name)
+      self.cursor_execute('SHOW CREATE TABLE %s' % table_name)
       table_data[table_name]['schema'] = self.cursor.fetchone()['Create Table']
-      self.cursor.execute('DESCRIBE %s' % table_name)
+      self.cursor_execute('DESCRIBE %s' % table_name)
       table_data[table_name]['columns'] = []
       table_descriptions = self.cursor.fetchall()
       for table_description in table_descriptions:
         table_data[table_name]['columns'].append(table_description['Field'])
-      self.cursor.execute('SELECT %s FROM %s' % 
+      self.cursor_execute('SELECT %s FROM %s' % 
                           (','.join(table_data[table_name]['columns']),
                            table_name))
       table_rows = self.cursor.fetchall()
@@ -789,6 +825,9 @@ class dbAccess(object):
         row_dict = {}
         for k,v in row.iteritems():
           row_dict[k] = self.connection.literal(v)
+          if( isinstance(row_dict[k], str) ):
+            row_dict[k] = unicode(row_dict[k],'utf-8')
+            
 
         table_data[table_name]['rows'].append(row_dict)
 
@@ -860,7 +899,7 @@ class dbAccess(object):
                                   'reverse_range_permissions',
                                   reverse_range_permissions_dict))
       if( not db_data ):
-        self.cursor.execute('SELECT access_level FROM users '
+        self.cursor_execute('SELECT access_level FROM users '
                             'WHERE user_name="%s"' % user)
         db_data.extend(self.cursor.fetchall())
 
@@ -900,9 +939,9 @@ class dbAccess(object):
               {'cidr_block': row['reverse_range_permissions_cidr_block'],
                'access_right': row['reverse_range_permissions_access_right']})
       else:
-        raise UnexpectedDataError('Row did not contain '
-                                  'reverse_range_permissions_access_right or '
-                                  'forward_zone_permissions_access_right.')
+        raise errors.UnexpectedDataError(
+            'Row did not contain reverse_range_permissions_access_right or '
+            'forward_zone_permissions_access_right.')
     return auth_info_dict
 
   def GetZoneOrigin(self, zone_name, view_name):
