@@ -32,14 +32,53 @@
 
 __copyright__ = 'Copyright (C) 2009, Purdue University'
 __license__ = 'BSD'
-__version__ = '0.14'
+__version__ = '0.15'
 
 
 class CoreError(Exception):
   pass
 
+class AuthError(CoreError):
+  pass
+
+class ConfigError(CoreError):
+  pass
+
 class DbAccessError(CoreError):
   pass
 
+class InvalidInputError(CoreError):
+  pass
+
+class InvalidInputError(DbAccessError):
+  pass
+
+class IPIndexError(CoreError):
+  pass
+
+class MaintenanceError(CoreError):
+  pass
+
+class MissingDataTypeError(DbAccessError):
+  pass
+
+class ReservedWordError(DbAccessError):
+  pass
+
+class RecordError(CoreError):
+  pass
+
+class RecordsBatchError(CoreError):
+  pass
+
+class TransactionError(DbAccessError):
+  pass
+
+class UnexpectedDataError(DbAccessError):
+  pass
+
+class UserError(CoreError):
+  pass
+
 
 # vi: set ai aw sw=2
